# -*- coding: utf-8 -*-
import gzip
import urllib2
import json
import re
from StringIO import StringIO
import xbmcaddon
import xbmc

acesrv = "http://%s/" % xbmcaddon.Addon().getSetting('aceserver')

user_agent = 'Mozilla/5.0 (Windows; U; Windows NT 5.1; en-GB; rv:1.9.0.3) Gecko/2008092417 Firefox/3.0.3'
acelive_url = "http://content.asplaylist.net/cdn/{}.acelive&hlc=1&spv=0" \
              "&transcode_audio=0&transcode_mp3=0&transcode_ac3=0&preferred_audio_language=eng"
acestream_url = 'http://ace-stream.tv/as.json'
tuchkatv_url = 'http://tuchkatv.org/player.html'
acetv_url = 'http://acetv.org/js/data.json'


def log(*s):
    if xbmcaddon.Addon().getSetting('debug') == 'true':
        xbmc.log(str(u'TVGyvai ' + u': ' + u': '.join([unicode(x) for x in s])), xbmc.LOGNOTICE)


def download(url):
    req = urllib2.Request(url)
    log('Downloading', url)
    req.add_header('User-Agent', user_agent)
    response = urllib2.urlopen(req, timeout=120)
    if response.info().get('Content-Encoding') == 'gzip':
        buf = StringIO(response.read())
        f = gzip.GzipFile(fileobj=buf)
        data = f.read()
    else:
        data = response.read()
    response.close()
    encoding = response.info().get('Content-Type').split('charset=')[-1]
    if encoding:
        try:
            data = unicode(data, encoding)
        except LookupError:
            pass
    return data


class Menu:
    def __init__(self, name, items=None, icon=None):
        self.label = name
        self.icon = icon
        self.image = None
        self.items = items or []


class Source:
    def __init__(self, name, link, icon=None):
        self.label = name
        # self.link = urllib.quote(link)
        self.link = link
        self.icon = icon


class AceUrlVideo(Source):
    def __init__(self, name, link, icon=None):
        Source.__init__(self, name, acesrv + 'ace/getstream?url=' + link, icon)


class AceIdVideo(Source):
    def __init__(self, name, link, icon=None):
        self.acelink = link
        Source.__init__(self, name, acesrv + 'ace/manifest.m3u8?id=' + link, icon)
        # self.getinfotag()

    def getinfotag(self):
        info = json.loads(download(acesrv + 'ace/manifest.m3u8?id=' + self.acelink + '&format=json'))
        stat_url = info.get(u'response', {}).get(u'stat_url')
        log('AceIdVideo', 'info ' + self.acelink, str(info))
        stat = json.loads(download(stat_url + '?format=json'))
        log('AceIdVideo', 'stat ' + self.acelink, str(stat))
        peers = stat.get(u'response', {}).get(u'peers', -1)
        if peers >= 0:
            self.label += ' [COLOR red][%d žiuri][/COLOR]' % peers
        else:
            self.label += ' [COLOR red][Aktyvus][/COLOR]'


class AceLiveVideo(AceUrlVideo):
    def __init__(self, name, link, icon=None):
        AceUrlVideo.__init__(self, name, acelive_url.format(link), icon)


def load_acestreamtv():
    return load_channel_json(acestream_url)


def load_acetv():
    return load_channel_json(acetv_url)


def load_channel_json(url):
    data = download(url)
    channels = json.loads(data)
    res = []
    chs = {}
    for ch in channels[u'channels']:
        cat = ch[u'cat'].lower().replace('_', ' ').capitalize()
        if cat in chs:
            chs[cat].append(ch)
        else:
            chs[cat] = [ch]
    for n, chl in chs.items():
        res.append(Menu(n, sorted([
            AceIdVideo(ch[u'name'], ch[u'url']) for ch in chl
        ], key=lambda x: x.label)))
    res = sorted(res, key=lambda x: x.label)
    res = [Menu(u'Visi', [AceIdVideo(ch[u'name'], ch[u'url']) for ch in channels[u'channels']])] + res

    return res


def load_tuchkatv():
    data = download(tuchkatv_url)
    channels = re.findall(r'<option.+value="([abcdef0-9]{40})".*>(.+)<\/option>', data, re.IGNORECASE)
    res = []
    for url, name in channels:
        name = name.replace(u'</a>', u'')
        res.append(AceIdVideo(name, url))
    return res


def lttv():
    return [
        # Source("LNK", "http://asdas545.ddns.net:7891"),
        # Source("BTV", "http://asdas545.ddns.net:9632"),
        # Source("Lietuvos Ryto TV", "http://asdas545.ddns.net:1241"),
        # Source("TV1", "http://asdas545.ddns.net:1238"),
        # Source("Info TV", "http://asdas545.ddns.net:1239"),
        # Source("Sport 1", "http://asdas545.ddns.net:1285"),
        AceIdVideo("LNK", "74e7f37525aa99fd42e5e04c3f7187c59bf31c9e"),
        AceLiveVideo("LRT KULTURA", "18528"),
    ]


main_menu = [
    Menu("Lietuviška TV", lttv),
    Menu("Ace-stream.tv", load_acestreamtv),
    Menu("Ace TV", load_acetv),
    Menu("Tuchka TV", load_tuchkatv),
]
